@@ -6,7 +6,6 @@ import java.util.Iterator;
 import java.util.List;
 
 import org.herac.tuxguitar.io.base.TGFileFormat;
-import org.herac.tuxguitar.io.base.TGInputStreamBase;
 import org.herac.tuxguitar.song.models.TGBeat;
 import org.herac.tuxguitar.song.models.TGChannel;
 import org.herac.tuxguitar.song.models.TGChord;
@@ -19,11 +18,13 @@ import org.herac.tuxguitar.song.models.TGNote;
 import org.herac.tuxguitar.song.models.TGNoteEffect;
 import org.herac.tuxguitar.song.models.TGSong;
 import org.herac.tuxguitar.song.models.TGString;
+import org.herac.tuxguitar.song.models.TGStroke;
 import org.herac.tuxguitar.song.models.TGTempo;
 import org.herac.tuxguitar.song.models.TGText;
 import org.herac.tuxguitar.song.models.TGTimeSignature;
 import org.herac.tuxguitar.song.models.TGTrack;
 import org.herac.tuxguitar.song.models.TGVelocities;
+import org.herac.tuxguitar.song.models.TGVoice;
 import org.herac.tuxguitar.song.models.effects.TGEffectBend;
 import org.herac.tuxguitar.song.models.effects.TGEffectGrace;
 import org.herac.tuxguitar.song.models.effects.TGEffectHarmonic;
@@ -34,15 +35,15 @@ import org.herac.tuxguitar.song.models.effects.TGEffectTremoloBar;
  * 
  * TODO To change the template for this generated type comment go to Window - Preferences - Java - Code Style - Code Templates
  */
-public class GP3InputStream extends GTPInputStream implements TGInputStreamBase {
+public class GP3InputStream extends GTPInputStream {
 	private static final String SUPPORTED_VERSIONS[] = new String[]{ "FICHIER GUITAR PRO v3.00" };
 	private static final float GP_BEND_SEMITONE = 25f;
 	private static final float GP_BEND_POSITION = 60f;
 	
 	private int tripletFeel;
 	
-	public GP3InputStream(){
-		super(SUPPORTED_VERSIONS);
+	public GP3InputStream(GTPSettings settings){
+		super(settings, SUPPORTED_VERSIONS);
 	}
 	
 	public TGFileFormat getFileFormat(){
@@ -92,8 +93,6 @@ public class GP3InputStream extends GTPInputStream implements TGInputStreamBase
 			channel.setReverb(toChannelShort(readByte()));
 			channel.setPhaser(toChannelShort(readByte()));
 			channel.setTremolo(toChannelShort(readByte()));
-			channel.setSolo(false);
-			channel.setMute(false);
 			channels.add(channel);
 			skip(2);
 		}
@@ -197,8 +196,9 @@ public class GP3InputStream extends GTPInputStream implements TGInputStreamBase
 				TGMeasure measure = track.getMeasure( m );
 				for (int b = measure.countBeats() - 1; b >= 0; b--) {
 					TGBeat beat = measure.getBeat( b );
-					for (int n = 0; n < beat.countNotes(); n ++) {
-						TGNote note = beat.getNote( n );
+					TGVoice voice = beat.getVoice(0);  
+					for (int n = 0; n < voice.countNotes(); n ++) {
+						TGNote note = voice.getNote( n );
 						if (note.getString() == string) {
 							return note.getValue();
 						}
@@ -271,6 +271,7 @@ public class GP3InputStream extends GTPInputStream implements TGInputStreamBase
 		}
 		
 		TGBeat beat = getFactory().newBeat();
+		TGVoice voice = beat.getVoice(0);
 		TGDuration duration = readDuration(flags);
 		TGNoteEffect effect = getFactory().newEffect();
 		if ((flags & 0x02) != 0) {
@@ -280,7 +281,7 @@ public class GP3InputStream extends GTPInputStream implements TGInputStreamBase
 			readText(beat);
 		}
 		if ((flags & 0x08) != 0) {
-			readBeatEffects(effect);
+			readBeatEffects(beat,effect);
 		}
 		if ((flags & 0x10) != 0) {
 			readMixChange(tempo);
@@ -290,11 +291,12 @@ public class GP3InputStream extends GTPInputStream implements TGInputStreamBase
 			if ((stringFlags & (1 << i)) != 0 && (6 - i) < track.stringCount()) {
 				TGString string = track.getString( (6 - i) + 1 ).clone(getFactory());
 				TGNote note = readNote(string, track,effect.clone(getFactory()));
-				beat.addNote(note);
+				voice.addNote(note);
 			}
 		}
 		beat.setStart(start);
-		duration.copy(beat.getDuration());
+		voice.setEmpty(false);
+		duration.copy(voice.getDuration());
 		measure.addBeat(beat);
 		
 		return duration.getTime();
@@ -492,7 +494,7 @@ public class GP3InputStream extends GTPInputStream implements TGInputStreamBase
 		}
 	}
 	
-	private void readBeatEffects(TGNoteEffect effect) throws IOException {
+	private void readBeatEffects(TGBeat beat,TGNoteEffect effect) throws IOException {
 		int flags = readUnsignedByte();
 		effect.setVibrato(((flags & 0x01) != 0) || ((flags & 0x02) != 0));
 		effect.setFadeIn(((flags & 0x10) != 0));
@@ -508,8 +510,15 @@ public class GP3InputStream extends GTPInputStream implements TGInputStreamBase
 			}
 		}
 		if ((flags & 0x40) != 0) {
-			readByte();
-			readByte();
+			int strokeDown = readByte();
+			int strokeUp = readByte();
+			if( strokeDown > 0 ){
+				beat.getStroke().setDirection( TGStroke.STROKE_DOWN );
+				beat.getStroke().setValue( toStrokeValue(strokeDown) );
+			}else if( strokeUp > 0 ){
+				beat.getStroke().setDirection( TGStroke.STROKE_UP );
+				beat.getStroke().setValue( toStrokeValue(strokeUp) );
+			}
 		}
 		if ((flags & 0x04) != 0) {
 			TGEffectHarmonic harmonic = getFactory().newEffectHarmonic();
@@ -557,6 +566,25 @@ public class GP3InputStream extends GTPInputStream implements TGInputStreamBase
 		}
 	}
 	
+	private int toStrokeValue( int value ){
+		if( value == 1 || value == 2){
+			return TGDuration.SIXTY_FOURTH;
+		}
+		if( value == 3){
+			return TGDuration.THIRTY_SECOND;
+		}
+		if( value == 4){
+			return TGDuration.SIXTEENTH;
+		}
+		if( value == 5){
+			return TGDuration.EIGHTH;
+		}
+		if( value == 6){
+			return TGDuration.QUARTER;
+		}
+		return TGDuration.SIXTY_FOURTH;
+	}
+	
 	private short toChannelShort(byte b){
 		short value = (short)(( b * 8 ) - 1);
 		return (short)Math.max(value,0);
