@@ -2,11 +2,10 @@ package org.herac.tuxguitar.io.gtp;
 
 import java.io.IOException;
 import java.util.ArrayList;
+import java.util.Iterator;
 import java.util.List;
 
 import org.herac.tuxguitar.io.base.TGFileFormat;
-import org.herac.tuxguitar.io.base.TGInputStreamBase;
-import org.herac.tuxguitar.song.factory.TGFactory;
 import org.herac.tuxguitar.song.models.TGBeat;
 import org.herac.tuxguitar.song.models.TGChannel;
 import org.herac.tuxguitar.song.models.TGChord;
@@ -20,11 +19,13 @@ import org.herac.tuxguitar.song.models.TGNote;
 import org.herac.tuxguitar.song.models.TGNoteEffect;
 import org.herac.tuxguitar.song.models.TGSong;
 import org.herac.tuxguitar.song.models.TGString;
+import org.herac.tuxguitar.song.models.TGStroke;
 import org.herac.tuxguitar.song.models.TGTempo;
 import org.herac.tuxguitar.song.models.TGText;
 import org.herac.tuxguitar.song.models.TGTimeSignature;
 import org.herac.tuxguitar.song.models.TGTrack;
 import org.herac.tuxguitar.song.models.TGVelocities;
+import org.herac.tuxguitar.song.models.TGVoice;
 import org.herac.tuxguitar.song.models.effects.TGEffectBend;
 import org.herac.tuxguitar.song.models.effects.TGEffectGrace;
 import org.herac.tuxguitar.song.models.effects.TGEffectHarmonic;
@@ -32,13 +33,13 @@ import org.herac.tuxguitar.song.models.effects.TGEffectTremoloBar;
 import org.herac.tuxguitar.song.models.effects.TGEffectTremoloPicking;
 import org.herac.tuxguitar.song.models.effects.TGEffectTrill;
 
-public class GP5InputStream extends GTPInputStream implements TGInputStreamBase {
+public class GP5InputStream extends GTPInputStream {
 	private static final String supportedVersions[] = { "FICHIER GUITAR PRO v5.00","FICHIER GUITAR PRO v5.10"};
 	private static final float GP_BEND_SEMITONE = 25f;
 	private static final float GP_BEND_POSITION = 60f;
 	
-	public GP5InputStream() {
-		super(supportedVersions);
+	public GP5InputStream(GTPSettings settings) {
+		super(settings, supportedVersions);
 	}
 	
 	public TGFileFormat getFileFormat(){
@@ -158,13 +159,15 @@ public class GP5InputStream extends GTPInputStream implements TGInputStreamBase
 		return lyric;
 	}
 	
-	private long readBeat(long start, TGMeasure measure, TGTrack track, TGTempo tempo) throws IOException{
+	private long readBeat(long start, TGMeasure measure, TGTrack track, TGTempo tempo, int voiceIndex) throws IOException{
 		int flags = readUnsignedByte();
-		if((flags & 0x40) != 0){
-			readUnsignedByte();
-		}
 		
-		TGBeat beat = getFactory().newBeat();
+		TGBeat beat = getBeat(measure, start);
+		TGVoice voice = beat.getVoice(voiceIndex);
+		if((flags & 0x40) != 0){
+			int beatType = readUnsignedByte();
+			voice.setEmpty( ( beatType & 0x02 ) == 0 );
+		}
 		TGDuration duration = readDuration(flags);
 		TGNoteEffect effect = getFactory().newEffect();
 		if ((flags & 0x02) != 0) {
@@ -174,7 +177,7 @@ public class GP5InputStream extends GTPInputStream implements TGInputStreamBase
 			readText(beat);
 		}
 		if ((flags & 0x08) != 0) {
-			readBeatEffects(effect);
+			readBeatEffects(beat, effect);
 		}
 		if ((flags & 0x10) != 0) {
 			readMixChange(tempo);
@@ -184,8 +187,9 @@ public class GP5InputStream extends GTPInputStream implements TGInputStreamBase
 			if ((stringFlags & (1 << i)) != 0 && (6 - i) < track.stringCount()) {
 				TGString string = track.getString( (6 - i) + 1 ).clone(getFactory());
 				TGNote note = readNote(string,track,effect.clone(getFactory()));
-				beat.addNote(note);
+				voice.addNote(note);
 			}
+			duration.copy(voice.getDuration());
 		}
 		
 		skip(1);
@@ -195,11 +199,7 @@ public class GP5InputStream extends GTPInputStream implements TGInputStreamBase
 			skip(1);
 		}
 		
-		beat.setStart(start);
-		duration.copy(beat.getDuration());
-		measure.addBeat(beat);
-		
-		return duration.getTime();
+		return (!voice.isEmpty() ? duration.getTime() : 0 );
 	}
 	
 	private List readChannels() throws IOException{
@@ -215,8 +215,6 @@ public class GP5InputStream extends GTPInputStream implements TGInputStreamBase
 			channel.setReverb(toChannelShort(readByte()));
 			channel.setPhaser(toChannelShort(readByte()));
 			channel.setTremolo(toChannelShort(readByte()));
-			channel.setSolo(false);
-			channel.setMute(false);
 			channels.add(channel);
 			skip(2);
 		}
@@ -280,10 +278,15 @@ public class GP5InputStream extends GTPInputStream implements TGInputStreamBase
 				TGMeasure measure = track.getMeasure( m );
 				for (int b = measure.countBeats() - 1; b >= 0; b--) {
 					TGBeat beat = measure.getBeat( b );
-					for (int n = 0; n < beat.countNotes(); n ++) {
-						TGNote note = beat.getNote( n );
-						if (note.getString() == string) {
-							return note.getValue();
+					for (int v = 0; v < beat.countVoices(); v ++) {
+						TGVoice voice = beat.getVoice(v);
+						if(!voice.isEmpty()){
+							for (int n = 0; n < voice.countNotes(); n ++) {
+								TGNote note = voice.getNote( n );
+								if (note.getString() == string) {
+									return note.getValue();
+								}
+							}
 						}
 					}
 				}
@@ -334,12 +337,12 @@ public class GP5InputStream extends GTPInputStream implements TGInputStreamBase
 			readByte();
 			readByte();
 		}
-		if ((flags & 0x10) == 0) {
-			skip(1);	
-		}
 		if ((flags & 0x01) != 0) {
 			skip(4);
 		}
+		if ((flags & 0x10) == 0) {
+			skip(1);
+		}
 		int tripletFeel = readByte();
 		if(tripletFeel == 1){
 			header.setTripletFeel(TGMeasureHeader.TRIPLET_FEEL_EIGHTH);
@@ -352,22 +355,32 @@ public class GP5InputStream extends GTPInputStream implements TGInputStreamBase
 	}
 	
 	private void readMeasure(TGMeasure measure, TGTrack track, TGTempo tempo) throws IOException {
-		//voice 1
-		long start = measure.getStart();
-		int beats = readInt();
-		for (int i = 0; i < beats; i++) {
-			start += readBeat(start, measure, track, tempo);
+		for( int voice = 0 ; voice < 2 ; voice ++ ){
+			long start = measure.getStart();
+			int beats = readInt();
+			for (int i = 0; i < beats; i++) {
+				start += readBeat(start, measure, track, tempo, voice);
+			}
 		}
 		
-		//voice 2
-		start = measure.getStart();
-		beats = readInt();
-		for (int i = 0; i < beats; i++) {
-			start += readBeat(start, measure, track, tempo);
+		List emptyBeats = new ArrayList();
+		for( int i = 0 ; i < measure.countBeats() ; i ++ ){
+			TGBeat beat = measure.getBeat( i );
+			boolean empty = true;
+			for( int v = 0 ; v < beat.countVoices() ; v ++ ){
+				if( !beat.getVoice( v ).isEmpty() ){
+					empty = false;
+				}
+			}
+			if( empty ){
+				emptyBeats.add( beat );
+			}
+		}
+		Iterator it = emptyBeats.iterator();
+		while( it.hasNext() ){
+			TGBeat beat = (TGBeat)it.next();
+			measure.removeBeat( beat );
 		}
-		
-		//join voices
-		new JoinVoicesHelper(getFactory(),measure).process();
 	}
 	
 	private TGNote readNote(TGString string,TGTrack track,TGNoteEffect effect)throws IOException {
@@ -468,7 +481,7 @@ public class GP5InputStream extends GTPInputStream implements TGInputStreamBase
 		}
 	}
 	
-	private void readBeatEffects(TGNoteEffect noteEffect) throws IOException {
+	private void readBeatEffects(TGBeat beat, TGNoteEffect noteEffect) throws IOException {
 		int flags1 = readUnsignedByte();
 		int flags2 = readUnsignedByte();
 		noteEffect.setFadeIn(((flags1 & 0x10) != 0));
@@ -483,8 +496,15 @@ public class GP5InputStream extends GTPInputStream implements TGInputStreamBase
 			readTremoloBar(noteEffect);
 		}
 		if ((flags1 & 0x40) != 0) {
-			readByte();
-			readByte();
+			int strokeUp = readByte();
+			int strokeDown = readByte();
+			if( strokeUp > 0 ){
+				beat.getStroke().setDirection( TGStroke.STROKE_UP );
+				beat.getStroke().setValue( toStrokeValue(strokeUp) );
+			}else if( strokeDown > 0 ){
+				beat.getStroke().setDirection( TGStroke.STROKE_DOWN );
+				beat.getStroke().setValue( toStrokeValue(strokeDown) );
+			} 
 		}
 		if ((flags2 & 0x02) != 0) {
 			readByte();
@@ -683,102 +703,41 @@ public class GP5InputStream extends GTPInputStream implements TGInputStreamBase
 		}
 	}
 	
+	private int toStrokeValue( int value ){
+		if( value == 1 || value == 2){
+			return TGDuration.SIXTY_FOURTH;
+		}
+		if( value == 3){
+			return TGDuration.THIRTY_SECOND;
+		}
+		if( value == 4){
+			return TGDuration.SIXTEENTH;
+		}
+		if( value == 5){
+			return TGDuration.EIGHTH;
+		}
+		if( value == 6){
+			return TGDuration.QUARTER;
+		}
+		return TGDuration.SIXTY_FOURTH;
+	}
+	
 	private short toChannelShort(byte b){
 		short value = (short)(( b * 8 ) - 1);
 		return (short)Math.max(value,0);
 	}
-}
-class JoinVoicesHelper{
-	private TGFactory factory;
-	private TGMeasure measure;
 	
-	public JoinVoicesHelper(TGFactory factory,TGMeasure measure){
-		this.factory = factory;
-		this.measure = measure;
-	}
-	
-	public void process(){
-		orderBeats();
-		joinBeats();
-	}
-	
-	public void joinBeats(){
-		TGBeat previous = null;
-		boolean finish = true;
-		
-		long measureStart = this.measure.getStart();
-		long measureEnd = (measureStart + this.measure.getLength());
-		for(int i = 0;i < this.measure.countBeats();i++){
-			TGBeat beat = this.measure.getBeat( i );
-			long beatStart = beat.getStart();
-			long beatLength = beat.getDuration().getTime();
-			if(previous != null){
-				long previousStart = previous.getStart();
-				long previousLength = previous.getDuration().getTime();
-				
-				if(previousStart == beatStart){
-					// add beat notes to previous
-					for(int n = 0;n < beat.countNotes();n++){
-						TGNote note = beat.getNote( n );
-						previous.addNote( note );
-					}
-					
-					// add beat chord to previous
-					if(!previous.isChordBeat() && beat.isChordBeat()){
-						previous.setChord( beat.getChord() );
-					}
-					
-					// add beat text to previous
-					if(!previous.isTextBeat() && beat.isTextBeat()){
-						previous.setText( beat.getText() );
-					}
-					
-					// set the best duration
-					if(beatLength > previousLength && (beatStart + beatLength) <= measureEnd){
-						beat.getDuration().copy(previous.getDuration());
-					}
-					
-					this.measure.removeBeat(beat);
-					finish = false;
-					break;
-				}
-				
-				else if(previousStart < beatStart && (previousStart + previousLength) > beatStart){
-					if(beat.isRestBeat()){
-						this.measure.removeBeat(beat);
-						finish = false;
-						break;
-					}
-					TGDuration duration = TGDuration.fromTime(this.factory, (beatStart - previousStart) );
-					duration.copy( previous.getDuration() );
-				}
+	private TGBeat getBeat(TGMeasure measure, long start){
+		int count = measure.countBeats();
+		for(int i = 0 ; i < count ; i ++ ){
+			TGBeat beat = measure.getBeat( i );
+			if( beat.getStart() == start ){
+				return beat;
 			}
-			if( (beatStart + beatLength) > measureEnd ){
-				if(beat.isRestBeat()){
-					this.measure.removeBeat(beat);
-					finish = false;
-					break;
-				}
-				TGDuration duration = TGDuration.fromTime(this.factory, (measureEnd - beatStart) );
-				duration.copy( beat.getDuration() );
-			}
-			previous = beat;
-		}
-		if(!finish){
-			joinBeats();
-		}
-	}
-	
-	public void orderBeats(){
-		for(int i = 0;i < this.measure.countBeats();i++){
-			TGBeat minBeat = null;
-			for(int j = i;j < this.measure.countBeats();j++){
-				TGBeat beat = this.measure.getBeat(j);
-				if(minBeat == null || beat.getStart() < minBeat.getStart()){
-					minBeat = beat;
-				}
-			}
-			this. measure.moveBeat(i, minBeat);
 		}
+		TGBeat beat = getFactory().newBeat();
+		beat.setStart(start);
+		measure.addBeat(beat);
+		return beat;
 	}
 }
