@@ -1,17 +1,16 @@
 package org.herac.tuxguitar.gui.editors.tab;
 
-import java.util.Iterator;
+import java.util.ArrayList;
+import java.util.List;
 
 import org.herac.tuxguitar.gui.TuxGuitar;
 import org.herac.tuxguitar.gui.editors.TGPainter;
 import org.herac.tuxguitar.gui.editors.tab.layout.TrackSpacing;
 import org.herac.tuxguitar.gui.editors.tab.layout.ViewLayout;
-import org.herac.tuxguitar.gui.editors.tab.painters.TGNotePainter;
-import org.herac.tuxguitar.gui.editors.tab.painters.TGSilencePainter;
 import org.herac.tuxguitar.song.factory.TGFactory;
 import org.herac.tuxguitar.song.models.TGBeat;
-import org.herac.tuxguitar.song.models.TGDuration;
-import org.herac.tuxguitar.song.models.TGTupleto;
+import org.herac.tuxguitar.song.models.TGStroke;
+import org.herac.tuxguitar.song.models.TGVoice;
 
 public class TGBeatImpl extends TGBeat{
 	/**
@@ -56,7 +55,7 @@ public class TGBeatImpl extends TGBeat{
 		this.posX = posX;
 	}
 	
-	public int getWidth() {
+	public int getMinimumWidth() {
 		return this.width;
 	}
 	
@@ -115,14 +114,22 @@ public class TGBeatImpl extends TGBeat{
 		this.joinedType = joinedType;
 	}
 	
-	public void setPreviousBeat(TGBeatImpl previous){
+	public TGBeatImpl getPreviousBeat() {
+		return this.previous;
+	}
+
+	public void setPreviousBeat(TGBeatImpl previous) {
 		this.previous = previous;
 	}
-	
-	public void setNextBeat(TGBeatImpl next){
+
+	public TGBeatImpl getNextBeat() {
+		return this.next;
+	}
+
+	public void setNextBeat(TGBeatImpl next) {
 		this.next = next;
 	}
-	
+
 	public TGBeatGroup getBeatGroup() {
 		return this.group;
 	}
@@ -139,6 +146,14 @@ public class TGBeatImpl extends TGBeat{
 		return (getMeasureImpl().isPlaying(layout) && TuxGuitar.instance().getEditorCache().isPlaying(getMeasure(),this));
 	}
 	
+	public TGVoiceImpl getVoiceImpl(int index){
+		TGVoice voice = super.getVoice(index);
+		if(voice instanceof TGVoiceImpl){
+			return (TGVoiceImpl)voice;
+		}
+		return null;
+	}
+	
 	public void reset(){
 		this.maxNote = null;
 		this.minNote = null;
@@ -161,12 +176,16 @@ public class TGBeatImpl extends TGBeat{
 		if(!TuxGuitar.instance().getPlayer().isRunning()){
 			new Thread(new Runnable() {
 				public void run() {
-					TuxGuitar.instance().getPlayer().playBeat(getMeasure().getTrack(),getNotes());
+					List notes = new ArrayList();
+					for( int v = 0; v < countVoices(); v ++){
+						notes.addAll( getVoice(v).getNotes() );
+					}
+					TuxGuitar.instance().getPlayer().playBeat(getMeasure().getTrack(),notes);
 				}
 			}).start();
 		}
 	}
-	
+	/*
 	public void update(ViewLayout layout) {
 		if(!isRestBeat()){
 			this.joinedType = JOINED_TYPE_NONE_RIGHT;
@@ -179,7 +198,7 @@ public class TGBeatImpl extends TGBeat{
 			
 			//trato de unir con el componente anterior
 			if (this.previous != null && !this.previous.isRestBeat()) {
-				if (getMeasureImpl().canJoin(layout.getSongManager(),this, this.previous)) {
+				if (getMeasureImpl().canJoin(layout.getSongManager(),this.getVoiceImpl(0), this.previous.getVoiceImpl(0))) {
 					withPrev = true;
 					if (this.previous.getDuration().getValue() >= getDuration().getValue()) {
 						this.setJoin1(this.previous);
@@ -195,7 +214,7 @@ public class TGBeatImpl extends TGBeat{
 			
 			//trato de unir con el componente que le sigue
 			if (this.next != null && !this.next.isRestBeat() ) {
-				if (getMeasureImpl().canJoin(layout.getSongManager(),this, this.next)) {
+				if (getMeasureImpl().canJoin(layout.getSongManager(),this.getVoiceImpl(0), this.next.getVoiceImpl(0))) {
 					if (this.next.getDuration().getValue() >= getDuration().getValue()) {
 						this.setJoin2(this.next);
 						if (this.previous == null || this.previous.isRestBeat() || this.previous.getDuration().getValue() < getDuration().getValue()) {
@@ -215,7 +234,7 @@ public class TGBeatImpl extends TGBeat{
 				this.joinedType = JOINED_TYPE_NONE_LEFT;
 			}
 		}
-	}
+	}*/
 	/*
 	public void paint(ViewLayout layout,TGPainter painter, int fromX, int fromY) {
 		paint(layout, painter, fromX, fromY, false);
@@ -225,6 +244,22 @@ public class TGBeatImpl extends TGBeat{
 		if(!layout.isPlayModeEnabled() && (layout.getStyle() & ViewLayout.DISPLAY_SCORE) != 0 ){
 			paintExtraLines(painter, layout,fromX, fromY);
 		}
+		for(int v = 0; v < TGBeat.MAX_VOICES; v ++){
+			getVoiceImpl(v).paint(layout, painter, fromX, fromY);
+		}
+		if(!layout.isPlayModeEnabled()){
+			if(isChordBeat()){
+				TGChordImpl chord = (TGChordImpl)getChord();
+				chord.paint(layout,painter,fromX,fromY);
+			}
+			if(getStroke().getDirection() != TGStroke.STROKE_NONE){
+				paintStroke(layout, painter, fromX, fromY);
+			}
+		}
+		/*
+		if(!layout.isPlayModeEnabled() && (layout.getStyle() & ViewLayout.DISPLAY_SCORE) != 0 ){
+			paintExtraLines(painter, layout,fromX, fromY);
+		}
 		
 		if(isRestBeat()){
 			paintSilence(layout, painter, fromX, fromY);
@@ -244,8 +279,9 @@ public class TGBeatImpl extends TGBeat{
 				}
 			}
 		}
+		*/
 	}
-	
+	/*
 	//----silence
 	public void paintSilence(ViewLayout layout,TGPainter painter, int fromX, int fromY) {
 		int style = layout.getStyle();
@@ -325,7 +361,7 @@ public class TGBeatImpl extends TGBeat{
 			}
 		}
 	}
-	
+	*//*
 	public void setStyle(ViewLayout layout, TGPainter painter, boolean playMode){
 		if((layout.getStyle() & ViewLayout.DISPLAY_SCORE) != 0 ){
 			layout.setScoreSilenceStyle(painter, playMode);
@@ -333,7 +369,7 @@ public class TGBeatImpl extends TGBeat{
 			layout.setTabSilenceStyle(painter, playMode);
 		}
 	}
-	
+	*/
 	public void paintExtraLines(TGPainter painter,ViewLayout layout,int fromX, int fromY){
 		if(!isRestBeat()){
 			int scoreY = (fromY + getMeasureImpl().getTs().getPosition(TrackSpacing.POSITION_SCORE_MIDDLE_LINES));
@@ -369,6 +405,48 @@ public class TGBeatImpl extends TGBeat{
 		}
 	}
 	
+	public void paintStroke(ViewLayout layout,TGPainter painter, int fromX, int fromY){
+		int style = layout.getStyle();
+		float scale = layout.getScale();
+		float x = (fromX + getPosX() + getSpacing() + ( 12f * scale ));
+		float y1 = 0;
+		float y2 = 0;
+		if((style & ViewLayout.DISPLAY_SCORE) != 0){
+			float y = (fromY + getPaintPosition(TrackSpacing.POSITION_SCORE_MIDDLE_LINES)); 
+			y1 = (y + layout.getScoreLineSpacing());
+			y2 = (y + (getMeasureImpl().getTrackImpl().getScoreHeight() - layout.getScoreLineSpacing()));
+		}
+		if((style & ViewLayout.DISPLAY_TABLATURE) != 0){
+			float y = (fromY + getPaintPosition(TrackSpacing.POSITION_TABLATURE));
+			y1 = (y + layout.getStringSpacing());
+			y2 = (y + (getMeasureImpl().getTrackImpl().getTabHeight() - layout.getStringSpacing()));
+		}
+		else if((style & ViewLayout.DISPLAY_SCORE) != 0){
+			float y = (fromY + getPaintPosition(TrackSpacing.POSITION_SCORE_MIDDLE_LINES)); 
+			y1 = (y + layout.getScoreLineSpacing());
+			y2 = (y + (getMeasureImpl().getTrackImpl().getScoreHeight() - layout.getScoreLineSpacing()));
+		}else{
+			return;
+		}
+		if( getStroke().getDirection() == TGStroke.STROKE_UP ){
+			painter.initPath();
+			painter.moveTo( x, y1 );
+			painter.lineTo( x, y2 );
+			painter.lineTo( x - (2.0f * scale), y2 - (5.0f * scale));
+			painter.moveTo( x , y2 );
+			painter.lineTo( x + (2.0f * scale), y2 - (5.0f * scale));
+			painter.closePath();
+		}else if( getStroke().getDirection() == TGStroke.STROKE_DOWN ){
+			painter.initPath();
+			painter.moveTo( x, y2 );
+			painter.lineTo( x, y1 );
+			painter.lineTo( x - (2.0f * scale), y1 + (3.0f * scale));
+			painter.moveTo( x , y1 );
+			painter.lineTo( x + (2.0f * scale), y1 + (3.0f * scale));
+			painter.closePath();
+		}
+	}
+	/*
 	public void paintBeat(ViewLayout layout,TGPainter painter, int fromX, int fromY){
 		if(!isRestBeat() ){
 			int style = layout.getStyle();
@@ -555,7 +633,7 @@ public class TGBeatImpl extends TGBeat{
 		}
 		painter.closePath();
 	}
-	
+	*/
 	public int getPaintPosition(int index){
 		return getMeasureImpl().getTs().getPosition(index);
 	}
