@@ -10,7 +10,9 @@ import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
 
+import org.eclipse.swt.graphics.Color;
 import org.eclipse.swt.graphics.Point;
+import org.eclipse.swt.graphics.RGB;
 import org.herac.tuxguitar.gui.TuxGuitar;
 import org.herac.tuxguitar.gui.editors.TGPainter;
 import org.herac.tuxguitar.gui.editors.tab.layout.TrackSpacing;
@@ -23,6 +25,7 @@ import org.herac.tuxguitar.gui.helper.SyncThread;
 import org.herac.tuxguitar.song.managers.TGSongManager;
 import org.herac.tuxguitar.song.models.TGBeat;
 import org.herac.tuxguitar.song.models.TGChord;
+import org.herac.tuxguitar.song.models.TGColor;
 import org.herac.tuxguitar.song.models.TGDuration;
 import org.herac.tuxguitar.song.models.TGMeasure;
 import org.herac.tuxguitar.song.models.TGMeasureHeader;
@@ -126,16 +129,21 @@ public class TGMeasureImpl extends TGMeasure{
 	
 	private int minY;
 	
+	private int notEmptyVoices;
+	
 	private int notEmptyBeats;
 	
 	private int widthBeats = 0;
 	
-	private List beatGroups;
+	//private List beatGroups; //TODO: borrar
+	private List[] voiceGroups;
 	
 	private TGMeasureBuffer buffer;
 	
 	private boolean bufferCreated;
 	
+	private Color markerColor;
+	
 	private int lyricBeatIndex;
 	private int width;
 	
@@ -153,8 +161,12 @@ public class TGMeasureImpl extends TGMeasure{
 	
 	public TGMeasureImpl(TGMeasureHeader header) {
 		super(header);
-		this.beatGroups = new ArrayList();
 		this.registeredAccidentals = new boolean[11][7];
+		//this.beatGroups = new ArrayList();
+		this.voiceGroups = new List[TGBeat.MAX_VOICES];
+		for(int v = 0 ; v < TGBeat.MAX_VOICES; v ++){
+			this.voiceGroups[v] = new ArrayList();
+		}
 	}
 	
 	/**
@@ -162,7 +174,7 @@ public class TGMeasureImpl extends TGMeasure{
 	 */
 	public void create(ViewLayout layout) {
 		this.divisionLength = TGSongManager.getDivisionLength(getHeader());
-		this.resetEffects();
+		this.resetSpacing();
 		this.autoCompleteSilences(layout.getSongManager());
 		this.orderBeats(layout.getSongManager());
 		this.checkCompactMode(layout);
@@ -218,36 +230,29 @@ public class TGMeasureImpl extends TGMeasure{
 		TGChord previousChord = null;
 		TGDuration minDuration = null;
 		TGBeatImpl previousBeat = null;
-		TGBeatGroup group = null;
+		TGVoiceImpl[] previousVoices = new TGVoiceImpl[TGBeat.MAX_VOICES];
+		//TGBeatGroup group = null;
+		TGBeatGroup[] groups = new TGBeatGroup[TGBeat.MAX_VOICES];
+		
 		int style = layout.getStyle();
 		int minimunChordLength = 0;
-		boolean beatChanged = false;
+		//int[] minimumNote = new int[TGBeat.MAX_VOICES];
+		//int[] maximumNote = new int[TGBeat.MAX_VOICES];
+		//boolean beatChanged = false;
+		//boolean voiceChanged[] = new boolean[TGBeat.MAX_VOICES];
+		
+		boolean[] notEmptyVoicesChecked = new boolean[TGBeat.MAX_VOICES];
 		boolean chordEnabled = ((style & (ViewLayout.DISPLAY_CHORD_DIAGRAM | ViewLayout.DISPLAY_CHORD_NAME)) != 0);
 		this.widthBeats = 0;
 		this.notEmptyBeats = 0;
-		this.beatGroups.clear();
+		this.notEmptyVoices = 0;
+		for(int v = 0 ; v < TGBeat.MAX_VOICES; v ++){
+			this.voiceGroups[v].clear();
+		}
 		
 		for (int i = 0; i < countBeats(); i++) {
 			TGBeatImpl beat = (TGBeatImpl)getBeat(i);
 			beat.reset();
-			if (minDuration == null || beat.getDuration().getTime() <= minDuration.getTime()) {
-				minDuration = beat.getDuration();
-			}
-			beatChanged = true;
-			Iterator it = beat.getNotes().iterator();
-			while(it.hasNext()){
-				TGNoteImpl note = (TGNoteImpl)it.next();
-				
-				beat.check(note);
-				
-				if( ( group == null ) || (beatChanged && !canJoin(layout.getSongManager(),beat,previousBeat) ) ){
-					group = new TGBeatGroup();
-					this.beatGroups.add(group);
-				}
-				group.check(note,getClef());
-				note.setBeatGroup(group);
-				beatChanged = false;
-			}
 			
 			if(chordEnabled && beat.getChord() != null){
 				if(previousChord != null){
@@ -256,11 +261,123 @@ public class TGMeasureImpl extends TGMeasure{
 				}
 				previousChord = beat.getChord();
 			}
-			
-			makeBeat(layout,beat,previousBeat,group,chordEnabled);
+			boolean emptyBeat = true;
+			for( int v = 0; v < TGBeat.MAX_VOICES; v ++){
+				TGVoiceImpl voice = (TGVoiceImpl)beat.getVoice(v);
+				if(!voice.isEmpty()){
+					emptyBeat = false;
+					
+					voice.reset();
+					if (minDuration == null || voice.getDuration().getTime() <= minDuration.getTime()) {
+						minDuration = voice.getDuration();
+					}
+					if( !notEmptyVoicesChecked[v] ){
+						notEmptyVoicesChecked[v] = true;
+						this.notEmptyVoices ++;
+					}
+					
+					//voiceChanged[v] = true;
+					Iterator it = voice.getNotes().iterator();
+					while(it.hasNext()){
+						TGNoteImpl note = (TGNoteImpl)it.next();
+						
+						voice.check(note);
+					}
+					
+					if(!voice.isRestVoice()){
+						beat.check(voice.getMinNote());
+						beat.check(voice.getMaxNote());
+						if( ( groups[v] == null ) || !canJoin(layout.getSongManager(),voice,previousVoices[v]) ){
+							groups[v] = new TGBeatGroup(v);
+							this.voiceGroups[v].add(groups[v]);
+						}
+						groups[v].check(voice);
+					}else{
+						for( int v2 = 0; v2 < TGBeat.MAX_VOICES; v2 ++){
+							if(v2 != voice.getIndex()){
+								TGVoiceImpl voice2 = beat.getVoiceImpl(v2);
+								if( !voice2.isEmpty() && voice2.getDuration().isEqual(voice.getDuration())){
+									if(!voice2.isRestVoice() || !voice2.isHiddenSilence()){
+										voice.setHiddenSilence(true);
+										break;
+									}
+								}
+							}
+						}
+					}
+					makeVoice(layout, voice, previousVoices[v], groups[v]);
+					previousVoices[v] = voice;
+				}
+			}
+			if (emptyBeat){
+				System.out.println( "Empty Beat !!!!!! " + beat.getStart() + "  " + i);
+			}
+			/*
+			for( int v = 0; v < TGBeat.MAX_VOICES; v ++){
+				TGVoiceImpl voice = (TGVoiceImpl)beat.getVoice(v);
+				if(!voice.isEmpty()){					
+					//
+				}
+			}
+			*/
+			/*
+			for( int v = 0; v < TGBeat.MAX_VOICES; v ++){
+				TGVoiceImpl voice = (TGVoiceImpl)beat.getVoice(v);
+				if(!voice.isEmpty()){
+					voice.reset();
+					if (minDuration == null || voice.getDuration().getTime() <= minDuration.getTime()) {
+						minDuration = voice.getDuration();
+					}
+					
+					voiceChanged[v] = true;
+					Iterator it = voice.getNotes().iterator();
+					while(it.hasNext()){
+						TGNoteImpl note = (TGNoteImpl)it.next();
+						
+						voice.check(note);
+						
+						if( ( groups[v] == null ) || (voiceChanged[v] && !canJoin(layout.getSongManager(),voice,previousVoices[v]) ) ){
+							groups[v] = new TGBeatGroup();
+							this.voiceGroups[v].add(groups[v]);
+						}
+						groups[v].check(note,getClef(), groups[ (v + 1) % groups.length ]);
+						note.setBeatGroup(groups[v]);
+						voiceChanged[v] = false;
+					}
+					
+					if(!voice.isRestVoice()){
+						beat.check(voice.getMinNote());
+						beat.check(voice.getMaxNote());
+					}else{
+						for( int v2 = 0; v2 < TGBeat.MAX_VOICES; v2 ++){
+							if(v2 != voice.getIndex()){
+								TGVoiceImpl voice2 = beat.getVoiceImpl(v2);
+								if( !voice2.isEmpty() && voice2.getDuration().isEqual(voice.getDuration())){
+									if(!voice2.isRestVoice() || !voice2.isHiddenSilence()){
+										voice.setHiddenSilence(true);
+										break;
+									}
+								}
+							}
+						}
+					}
+					makeVoice(layout, voice, previousVoices[v], groups[v]);
+					previousVoices[v] = voice;
+				}
+			}
+			*/
+			makeBeat(layout,beat,previousBeat,chordEnabled);
 			previousBeat = beat;
 		}
 		
+		for(int v = 0; v < this.voiceGroups.length; v ++){
+			Iterator voiceGroups = this.voiceGroups[v].iterator();
+			while (voiceGroups.hasNext()) {
+				TGBeatGroup group = (TGBeatGroup)voiceGroups.next();
+				group.finish(layout,this);
+			}
+		}
+		
 		if(!this.compactMode){
 			this.quarterSpacing = (minDuration != null)?layout.getSpacingForQuarter(minDuration): Math.round(DEFAULT_QUARTER_SPACING * layout.getScale());
 			if(chordEnabled && minimunChordLength > 0){
@@ -272,15 +389,15 @@ public class TGMeasureImpl extends TGMeasure{
 		}
 	}
 	
-	public boolean canJoin(TGSongManager manager,TGBeatImpl b1,TGBeatImpl b2){
-		if( b1 == null || b2 == null || b1.isRestBeat() || b2.isRestBeat() ){
+	public boolean canJoin(TGSongManager manager,TGVoiceImpl b1,TGVoiceImpl b2){
+		if( b1 == null || b2 == null || b1.isRestVoice() || b2.isRestVoice() ){
 			return false;
 		}
 		
 		long divisionLength = getDivisionLength();
 		long start = getStart();
-		long start1 = (manager.getMeasureManager().getRealStart(this, b1.getStart()) - start);
-		long start2 = (manager.getMeasureManager().getRealStart(this, b2.getStart()) - start);
+		long start1 = (manager.getMeasureManager().getRealStart(this, b1.getBeat().getStart()) - start);
+		long start2 = (manager.getMeasureManager().getRealStart(this, b2.getBeat().getStart()) - start);
 		
 		if(b1.getDuration().getValue() < TGDuration.EIGHTH || b2.getDuration().getValue() < TGDuration.EIGHTH ){
 			return ( start1 == start2);
@@ -292,22 +409,45 @@ public class TGMeasureImpl extends TGMeasure{
 		return  (   p1 == p2  );
 	}
 	
-	private void makeBeat(ViewLayout layout,TGBeatImpl beat,TGBeatImpl previousBeat,TGBeatGroup group,boolean chordEnabled){
-		beat.setWidth((int)layout.getBeatWidth(beat));
-		beat.setBeatGroup( group );
-		this.notEmptyBeats += (beat.isRestBeat() ? 0 : 1);
-		this.widthBeats += beat.getWidth();
+	private void makeVoice(ViewLayout layout,TGVoiceImpl voice,TGVoiceImpl previousVoice,TGBeatGroup group){
+		voice.setWidth((int)layout.getVoiceWidth(voice));
+		voice.setBeatGroup( group );
+		
+		if(previousVoice != null){
+			voice.setPreviousBeat(previousVoice);
+			previousVoice.setNextBeat(voice);
+		}
+	}
+	
+	private void makeBeat(ViewLayout layout,TGBeatImpl beat,TGBeatImpl previousBeat, boolean chordEnabled){
+		int minimumWidth = -1;
+		boolean restBeat = true;
+		for(int v = 0 ; v < TGBeat.MAX_VOICES; v ++){
+			TGVoiceImpl voice = beat.getVoiceImpl(v);
+			if(!voice.isEmpty()){
+				if( minimumWidth < 0 || voice.getWidth() < minimumWidth ){
+					minimumWidth = voice.getWidth();
+				}
+				if( !voice.isRestVoice() ){
+					restBeat = false;
+				}
+			}
+		}
+		
+		beat.setWidth( minimumWidth );
+		this.notEmptyBeats += (restBeat ? 0 : 1);
+		this.widthBeats += beat.getMinimumWidth();
 		
 		if(previousBeat != null){
 			beat.setPreviousBeat(previousBeat);
 			previousBeat.setNextBeat(beat);
 			
 			if(chordEnabled && beat.isChordBeat() && previousBeat.isChordBeat()){
-				int previousWidth = previousBeat.getWidth();
+				int previousWidth = previousBeat.getMinimumWidth();
 				int chordWidth = (layout.getChordFretIndexSpacing() + layout.getChordStringSpacing() + (getTrack().stringCount() * layout.getChordStringSpacing()));
 				previousBeat.setWidth(Math.max(chordWidth,previousWidth));
 				this.widthBeats -= previousWidth;
-				this.widthBeats += previousBeat.getWidth();
+				this.widthBeats += previousBeat.getMinimumWidth();
 			}
 		}
 	}
@@ -349,28 +489,57 @@ public class TGMeasureImpl extends TGMeasure{
 		int tmpX = spacing;
 		for (int i = 0; i < countBeats(); i++) {
 			TGBeatImpl beat = (TGBeatImpl) getBeat(i);
-			beat.update(layout);
+			
 			if(this.compactMode){
 				beat.setPosX(tmpX);
-				tmpX += beat.getWidth();
+				tmpX += beat.getMinimumWidth();
 			}
 			else{
 				int quarterWidth = getMaxQuarterSpacing(layout);
 				int x1 = (spacing + TablatureUtil.getStartPosition(this, beat.getStart(), quarterWidth));
-				int x2 = (spacing + TablatureUtil.getStartPosition(this, beat.getStart() + beat.getDuration().getTime(), quarterWidth));
+				int minimumWidth = -1;
+				for(int v = 0 ; v < beat.countVoices(); v ++){
+					TGVoiceImpl voice = beat.getVoiceImpl(v);
+					if(!voice.isEmpty()){
+						int x2 = (spacing + TablatureUtil.getStartPosition(this, beat.getStart() + voice.getDuration().getTime(), quarterWidth));
+						int width = ( x2 - x1 );
+						if( minimumWidth < 0 || width < minimumWidth ){
+							minimumWidth = width;
+						}
+						voice.setWidth( width );
+					}
+				}
 				beat.setPosX( x1 );
-				beat.setWidth( x2 - x1 );
+				beat.setWidth( minimumWidth );
 			}
 			
-			Iterator notes = beat.getNotes().iterator();
-			while(notes.hasNext()){
-				TGNoteImpl note = (TGNoteImpl)notes.next();
-				
-				checkEffects(layout,note.getEffect());
-				
-				note.update(layout);
+			for(int v = 0 ; v < beat.countVoices(); v ++){
+				TGVoiceImpl voice = beat.getVoiceImpl(v);
+				if(!voice.isEmpty()){
+					Iterator notes = voice.getNotes().iterator();
+					while(notes.hasNext()){
+						TGNoteImpl note = (TGNoteImpl)notes.next();
+						
+						checkEffects(layout,note.getEffect());
+						
+						note.update(layout);
+					}
+					
+					voice.update(layout);
+					
+					if(!this.tupleto && !voice.getDuration().getTupleto().isEqual(TGTupleto.NORMAL)){
+						this.tupleto = true;
+					}
+					if( (layout.getStyle() & ViewLayout.DISPLAY_SCORE) == 0 || (voice.isRestVoice() && !voice.isHiddenSilence()) ){
+						if( voice.getMaxY() > this.maxY ){
+							this.maxY = voice.getMaxY();
+						}
+						if( voice.getMinY() < this.minY ){
+							this.minY = voice.getMinY();
+						}
+					}
+				}
 			}
-			beat.update(layout);
 			
 			if(!this.chord && beat.isChordBeat()){
 				this.chord = true;
@@ -379,16 +548,17 @@ public class TGMeasureImpl extends TGMeasure{
 			if(!this.text && beat.isTextBeat()){
 				this.text = true;
 			}
-			
-			if(!this.tupleto && !beat.getDuration().getTupleto().isEqual(TGTupleto.NORMAL)){
-				this.tupleto = true;
-			}
 		}
-		Iterator groups = this.beatGroups.iterator();
-		while (groups.hasNext()) {
-			TGBeatGroup group = (TGBeatGroup)groups.next();
-			checkValue(layout,group.getMinNote(),group.getDirection());
-			checkValue(layout,group.getMaxNote(),group.getDirection());
+		
+		if( (layout.getStyle() & ViewLayout.DISPLAY_SCORE) != 0){
+			for(int i = 0; i < this.voiceGroups.length; i ++){
+				Iterator groups = this.voiceGroups[i].iterator();
+				while (groups.hasNext()) {
+					TGBeatGroup group = (TGBeatGroup)groups.next();
+					checkValue(layout,group.getMinNote(),group.getDirection());
+					checkValue(layout,group.getMaxNote(),group.getDirection());
+				}
+			}
 		}
 	}
 	
@@ -454,7 +624,7 @@ public class TGMeasureImpl extends TGMeasure{
 		}
 	}
 	
-	private void resetEffects(){
+	private void resetSpacing(){
 		this.text = false;
 		this.chord = false;
 		this.tupleto = false;
@@ -466,7 +636,10 @@ public class TGMeasureImpl extends TGMeasure{
 		this.vibrato = false;
 	}
 	
-	public void registerEffects(ViewLayout layout,TrackSpacing ts){
+	public void registerSpacing(ViewLayout layout,TrackSpacing ts){
+		if(this.hasMarker()){
+			ts.setSize(TrackSpacing.POSITION_MARKER,layout.getMarkerSpacing());
+		}
 		if(this.chord){
 			ts.setSize(TrackSpacing.POSITION_CHORD,layout.getDefaultChordSpacing());
 		}
@@ -528,6 +701,7 @@ public class TGMeasureImpl extends TGMeasure{
 			painter.setBackground(layout.getResources().getBackgroundColor());
 			getBuffer().paintImage(painter,getPosX(),getPosY(),getTs().getPosition(TrackSpacing.POSITION_BUFFER_SEPARATOR));
 		}
+		this.paintMarker(layout, painter);
 		this.paintTexts(layout,painter);
 		this.paintTempo(layout,painter);
 		this.paintTripletFeel(layout,painter);
@@ -919,6 +1093,17 @@ public class TGMeasureImpl extends TGMeasure{
 		}
 	}
 	
+	private void paintMarker(ViewLayout layout,TGPainter painter){
+		if( this.hasMarker() ){
+			int x = (getPosX() + getHeaderImpl().getLeftSpacing(layout) + getFirstNoteSpacing(layout));
+			int y = (getPosY() + getTs().getPosition(TrackSpacing.POSITION_MARKER));
+			
+			layout.setMarkerStyle(painter);
+			painter.setForeground( getMarkerColor() );
+			painter.drawString(getMarker().getTitle(), x, y);
+		}
+	}
+	
 	private void paintTexts(ViewLayout layout,TGPainter painter){
 		Iterator it = getBeats().iterator();
 		while(it.hasNext()){
@@ -1128,6 +1313,10 @@ public class TGMeasureImpl extends TGMeasure{
 		return this.notEmptyBeats;
 	}
 	
+	public int getNotEmptyVoices(){
+		return this.notEmptyVoices;
+	}
+	
 	public int getLyricBeatIndex() {
 		return this.lyricBeatIndex;
 	}
@@ -1155,11 +1344,33 @@ public class TGMeasureImpl extends TGMeasure{
 		return this.buffer;
 	}
 	
+	public Color getMarkerColor(){
+		TGColor color = getMarker().getColor();
+		if(this.markerColor != null && !this.markerColor.isDisposed()){
+			RGB rgb = this.markerColor.getRGB();
+			if( rgb.red != color.getR() || rgb.green != color.getG() ||  rgb.blue != color.getB()){
+				this.disposeMarkerColor();
+			}
+		}
+		if(this.markerColor == null || this.markerColor.isDisposed()){
+			this.markerColor = new Color(TuxGuitar.instance().getDisplay(), color.getR(),color.getG(),color.getB());
+		}
+		return this.markerColor;
+	}
+
+	public void disposeMarkerColor(){
+		if(this.markerColor != null && !this.markerColor.isDisposed()){
+			this.markerColor.dispose();
+			this.markerColor = null;
+		}
+	}
+	
 	public void dispose(){
 		new SyncThread( new Runnable() {
 			public void run() {
 				if(!TuxGuitar.isDisposed()){
 					getBuffer().dispose();
+					disposeMarkerColor();
 					Iterator it = getBeats().iterator();
 					while(it.hasNext()){
 						TGBeatImpl beat = (TGBeatImpl)it.next();
