@@ -27,11 +27,13 @@ import org.herac.tuxguitar.song.models.TGNote;
 import org.herac.tuxguitar.song.models.TGNoteEffect;
 import org.herac.tuxguitar.song.models.TGSong;
 import org.herac.tuxguitar.song.models.TGString;
+import org.herac.tuxguitar.song.models.TGStroke;
 import org.herac.tuxguitar.song.models.TGTempo;
 import org.herac.tuxguitar.song.models.TGText;
 import org.herac.tuxguitar.song.models.TGTimeSignature;
 import org.herac.tuxguitar.song.models.TGTrack;
 import org.herac.tuxguitar.song.models.TGTupleto;
+import org.herac.tuxguitar.song.models.TGVoice;
 import org.herac.tuxguitar.song.models.effects.TGEffectBend;
 import org.herac.tuxguitar.song.models.effects.TGEffectGrace;
 import org.herac.tuxguitar.song.models.effects.TGEffectHarmonic;
@@ -109,6 +111,12 @@ public class TGOutputStream extends TGStream implements TGOutputStreamBase{
 	private void writeTrack(TGTrack track){
 		//header
 		int header = 0;
+		if (track.isSolo()) {
+			header |= TRACK_SOLO;
+		}
+		if (track.isMute()) {
+			header |= TRACK_MUTE;
+		}
 		if(!track.getLyrics().isEmpty()){
 			header |= TRACK_LYRICS;
 		}
@@ -249,11 +257,6 @@ public class TGOutputStream extends TGStream implements TGOutputStreamBase{
 	}
 	
 	private void writeChannel(TGChannel channel){
-		int header = 0;
-		header = (channel.isSolo())?header |= CHANNEL_SOLO:header;
-		header = (channel.isMute())?header |= CHANNEL_MUTE:header;
-		writeHeader(header);
-		
 		//escribo el canal
 		writeByte(channel.getChannel());
 		
@@ -286,29 +289,47 @@ public class TGOutputStream extends TGStream implements TGOutputStreamBase{
 		int count = measure.countBeats();
 		for(int i = 0; i < count; i ++){
 			TGBeat beat = measure.getBeat(i);
-			writeComponent(beat,data, (i + 1 < count ));
+			writeBeat(beat,data, (i + 1 < count ));
 		}
 	}
 	
-	private void writeComponent(TGBeat beat,TGBeatData data, boolean hasNext){
+	private void writeBeat(TGBeat beat,TGBeatData data, boolean hasNext){
 		int header = hasNext ? BEAT_HAS_NEXT : 0;
 		
-		//Berifico si hay cambio de duracion
-		if(!beat.getDuration().isEqual(data.getDuration())){
-			header |= BEAT_NEXT_DURATION;
-			data.setDuration(beat.getDuration());
+		//Berifico si hay cambios en las voces
+		for(int i = 0 ; i < TGBeat.MAX_VOICES; i ++ ){
+			int shift = (i * 2 );
+			if(!beat.getVoice(i).isEmpty()){
+				header |= ( BEAT_HAS_VOICE << shift ); 
+				
+				int flags = ( beat.getVoice(i).isRestVoice() ? 0 : VOICE_HAS_NOTES );
+				if(!beat.getVoice(i).getDuration().isEqual(data.getVoice(i).getDuration())){
+					flags |= VOICE_NEXT_DURATION;
+					data.getVoice(i).setDuration(beat.getVoice(i).getDuration());
+				}
+				if(beat.getVoice(i).getDirection() != TGVoice.DIRECTION_NONE ){
+					if(beat.getVoice(i).getDirection() == TGVoice.DIRECTION_UP ){
+						flags |= VOICE_DIRECTION_UP;
+					}
+					else if(beat.getVoice(i).getDirection() == TGVoice.DIRECTION_DOWN ){
+						flags |= VOICE_DIRECTION_DOWN;
+					}
+				}
+				if( data.getVoice(i).getFlags() != flags ){
+					header |= ( BEAT_HAS_VOICE_CHANGES << shift ); 
+					data.getVoice(i).setFlags( flags );
+				}
+			}
+			
 		}
-		
-		//Berifico si tiene notas
-		if(!beat.isRestBeat()){
-			header |= BEAT_HAS_NOTES;
+		//Berifico si tiene stroke
+		if(beat.getStroke().getDirection() != TGStroke.STROKE_NONE){
+			header |= BEAT_HAS_STROKE;
 		}
-		
 		//Berifico si tiene acorde
 		if(beat.getChord() != null){
 			header |= BEAT_HAS_CHORD;
 		}
-		
 		//Berifico si tiene texto
 		if(beat.getText() != null){
 			header |= BEAT_HAS_TEXT;
@@ -317,14 +338,12 @@ public class TGOutputStream extends TGStream implements TGOutputStreamBase{
 		// escribo la cabecera
 		writeHeader(header);
 		
-		//escribo la duracion
-		if(((header & BEAT_NEXT_DURATION) != 0)){
-			writeDuration(beat.getDuration());
-		}
+		//escribo las voces
+		writeVoices(header, beat, data);
 		
-		//escribo las notas
-		if(((header & BEAT_HAS_NOTES) != 0)){
-			writeNotes(beat, data);
+		//escribo el stroke
+		if(((header & BEAT_HAS_STROKE) != 0)){
+			writeStroke(beat.getStroke());
 		}
 		
 		//escribo el acorde
@@ -338,14 +357,36 @@ public class TGOutputStream extends TGStream implements TGOutputStreamBase{
 		}
 	}
 	
-	private void writeNotes(TGBeat beat,TGBeatData data){
-		for( int i = 0 ; i < beat.countNotes() ; i ++){
-			TGNote note = beat.getNote(i);
+	private void writeVoices(int header, TGBeat beat,TGBeatData data){
+		for(int i = 0 ; i < TGBeat.MAX_VOICES; i ++ ){
+			int shift = (i * 2 );
+			if((( header & (BEAT_HAS_VOICE << shift)) != 0)){
+				
+				if(((header & (BEAT_HAS_VOICE_CHANGES << shift)) != 0)){
+					writeHeader( data.getVoice(i).getFlags() );
+				}
+				
+				//escribo la duracion
+				if((( data.getVoice(i).getFlags() & VOICE_NEXT_DURATION) != 0)){
+					writeDuration(beat.getVoice(i).getDuration());
+				}
+				
+				//escribo las notas
+				if((( data.getVoice(i).getFlags() & VOICE_HAS_NOTES) != 0)){
+					writeNotes(beat.getVoice(i), data);
+				}
+			}
+		}
+	}
+	
+	private void writeNotes(TGVoice voice,TGBeatData data){
+		for( int i = 0 ; i < voice.countNotes() ; i ++){
+			TGNote note = voice.getNote(i);
 			
-			int header = ( i + 1 < beat.countNotes() ? NOTE_HAS_NEXT : 0 );
+			int header = ( i + 1 < voice.countNotes() ? NOTE_HAS_NEXT : 0 );
 			header = (note.isTiedNote())?header |= NOTE_TIED:header;
-			if(note.getVelocity() != data.getVelocity()){
-				data.setVelocity(note.getVelocity());
+			if(note.getVelocity() != data.getVoice(voice.getIndex()).getVelocity()){
+				data.getVoice(voice.getIndex()).setVelocity(note.getVelocity());
 				header |= NOTE_VELOCITY;
 			}
 			header = (note.getEffect().hasAnyEffect())?header |= NOTE_EFFECT:header;
@@ -374,6 +415,14 @@ public class TGOutputStream extends TGStream implements TGOutputStreamBase{
 		}
 	}
 	
+	private void writeStroke(TGStroke stroke){
+		//escribo la direccion
+		writeByte(stroke.getDirection());
+		
+		//escribo el valor
+		writeByte(stroke.getValue());
+	}
+	
 	private void writeChord(TGChord chord){
 		//escribo la cantidad de cuerdas
 		writeByte(chord.countStrings());
